@@ -25,9 +25,9 @@ except ImportError:
 from pipeline.time_manager import TimeManager
 from pipeline.create_sensors_db import get_db_connection, create_packaged_db, reset_user_db # get_user_db_path, ensure_user_db, 
 from pipeline.api.eds import demo_eds_webplot_point_live, EdsClient, load_historic_data, EdsLoginException, demo_eds_save_point_export
-from pipeline.security_and_config import get_eds_api_credentials, get_external_api_credentials, get_eds_db_credentials, get_all_configured_urls, get_configurable_plant_name, init_security, CONFIG_PATH
-from pipeline.termux_setup import setup_termux_install, cleanup_termux_install
-from pipeline.windows_setup import setup_windows_install, cleanup_windows_install
+from pipeline.security_and_config import get_eds_rest_api_credentials, get_external_api_credentials, get_eds_local_db_credentials, get_all_configured_urls, get_configurable_default_plant_name, init_security, CONFIG_PATH
+from pipeline.termux_setup import setup_termux_integration, cleanup_termux_integration
+from pipeline.windows_setup import setup_windows_integration, cleanup_windows_integration
 from pipeline import helpers
 from pipeline.plotbuffer import PlotBuffer
 from pipeline.version_info import  PIP_PACKAGE_NAME, PIPELINE_VERSION, __version__, get_package_version, get_package_name
@@ -40,9 +40,9 @@ from pipeline.version_info import  PIP_PACKAGE_NAME, PIPELINE_VERSION, __version
 #-- SUPPRESS with "False and" as of 0.3.53 - automaic installation on every run is invasive and is annyoring for troubleshooting, like if the user changes the .shortcut/filename
 #-- user may directly run 'install' command
 if False and on_termux():
-    setup_termux_install()
+    setup_termux_integration()
 elif False and on_windows():
-    setup_windows_install()
+    setup_windows_integration()
 # --- end SETUP / INSTALL HOOK ---
 
 # -- Versioning --
@@ -136,7 +136,8 @@ def list_sensors(
 def live(
     idcs: list[str] = typer.Argument(..., help="Provide known idcs values that match the given zd."), # , "--idcs", "-i"
     zd: str = typer.Option('Maxson', "--zd", "-z", help = "Define the EDS ZD from your secrets file. This must correlate with your idcs point selection(s)."),
-    webplot: bool = typer.Option(False,"--webplot","-w",help = "Use a web-based plot (plotly) instead of matplotlib. Useful for remote servers without display.")
+    force_webplot: bool = typer.Option(False,"--webplot","-w",help = "Use a web-based plot (plotly) instead of matplotlib. Useful for remote servers without display."),
+    force_matplotlib: bool = typer.Option(False,"--matplotlib","-mpl",help = "Force matplotlib to be used for plotting. This will not work if matplotlib is not available.")
 ):
     """live data plotting, based on CSV query files. Coming soon - call any, like the 'trend' command."""
     typer.echo(f"Coming soon!")
@@ -147,7 +148,7 @@ def defaultplant(
     overwrite: bool = typer.Option(False, "--overwrite", "-o", help = "Overwrite existing plant name(s) to be used as default.")
     ):
     """Set the plant name(s) to be used if one is not explicitly provided in other commands, like trend. Comma separate for multiple."""
-    configurable_plant_name = get_configurable_plant_name(overwrite=overwrite)
+    configurable_plant_name = get_configurable_default_plant_name(overwrite=overwrite)
     typer.echo(f"Configurable plant name(s) for EDS API: {configurable_plant_name}")
 
 @app.command()
@@ -159,7 +160,8 @@ def trend(
     plant_name: str = typer.Option(None, "--plantname", "-pn", help = "Provide the EDS ZD for your credentials."),
     print_csv: bool = typer.Option(False,"--print-csv","-p",help = "Print the CSV style for pasting into Excel."),
     step_seconds: int = typer.Option(None, "--step-seconds", help="You can explicitly provide the delta between datapoints. If not, ~400 data points will be used, based on the nice_step() function."), 
-    webplot: bool = typer.Option(False,"--webplot","-w",help = "Use a browser-based plot instead of local (matplotlib). Useful for remote servers without display."),
+    force_webplot: bool = typer.Option(False,"--webplot","-w",help = "Use a browser-based plot instead of local (matplotlib). Useful for remote servers without display."),
+    force_matplotlib: bool = typer.Option(False,"--matplotlib","-mpl",help="Forcr matplotlib to be used for plotting. This will not work if matplotlib is not available."),
     default_idcs: bool = typer.Option(False, "--default-idcs", "-d", help="Use the default IDCS values for the configured plant name, instead of providing them as arguments.")
     ):
     """
@@ -168,15 +170,16 @@ def trend(
 
     #zd = api_credentials.get("zd")
     if plant_name is None:
-        plant_name = get_configurable_plant_name()
+        plant_name = get_configurable_default_plant_name()
 
     # --- Conditional IDCS Input ---
     if idcs is None:
         if default_idcs:
+            
             from pipeline.security_and_config import get_configurable_idcs_list
             # plant_name is resolved below, but we need a valid name for the helper
             # Temporarily resolve plant_name for the prompt if needed
-            current_plant_name = plant_name if plant_name is not None else get_configurable_plant_name()
+            current_plant_name = plant_name if plant_name is not None else get_configurable_default_plant_name()
             idcs = get_configurable_idcs_list(current_plant_name)
             
             if not idcs:
@@ -202,13 +205,13 @@ def trend(
     # Retrieve all necessary API credentials and config values.
     # This will prompt the user if any are missing.
     if isinstance(plant_name,str):
-        api_credentials = get_eds_api_credentials(plant_name=plant_name)
+        api_credentials = get_eds_rest_api_credentials(plant_name=plant_name)
     if isinstance(plant_name,list):
         typer.echo("")
         typer.echo(f"Multiple plant names provided: {plant_name} ")
         typer.echo("Querying multiple plants at once currently supported.") 
         typer.echo("Defaulting to use the first name.")
-        api_credentials = get_eds_api_credentials(plant_name=plant_name[0])
+        api_credentials = get_eds_rest_api_credentials(plant_name=plant_name[0])
 
     typer.echo(f"")
     typer.echo(f"Data request processing...")
@@ -264,12 +267,12 @@ def trend(
     # Once the loop is done, you can call your show_static function
     # with the single, populated data_buffer.
 
-    if webplot or not matplotlib_is_available_for_gui_plotting():
+    if force_webplot or not force_matplotlib or not matplotlib_is_available_for_gui_plotting():
         from pipeline import gui_plotly_static
         #from pipeline import gui_plotly_static_backup_06Oct25 as gui_plotly_static
         #gui_fastapi_plotly_live.run_gui(data_buffer)
         gui_plotly_static.show_static(data_buffer)
-    else:
+    elif matplotlib_is_available_for_gui_plotting():
         from pipeline import gui_mpl_live
         #gui_mpl_live.run_gui(data_buffer)
         gui_mpl_live.show_static(data_buffer)
@@ -332,11 +335,11 @@ def configure_credentials(
         
         # Configure API for this plant
         if typer.confirm(f"Do you want to configure the EDS API for '{name}'?", default=True):
-            get_eds_api_credentials(plant_name=name, overwrite=overwrite)
+            get_eds_rest_api_credentials(plant_name=name, overwrite=overwrite)
 
         # Configure DB for this plant
         if False and typer.confirm(f"Do you want to configure the EDS database for '{name}'?",  default=False):
-            get_eds_db_credentials(plant_name=name, overwrite=overwrite)
+            get_eds_local_db_credentials(plant_name=name, overwrite=overwrite)
     
     # Configure any other external APIs
     if False and typer.confirm("Do you want to configure external API credentials? (e.g., RJN)"):
@@ -382,22 +385,22 @@ def setup_integration(
     if uninstall:
         if on_windows():
             if typer.confirm("Are you sure you want to uninstall the registry context-menu item, the launcher BAT, and empty out the AppData folder?"):
-                cleanup_windows_install()
+                cleanup_windows_integration()
         elif on_termux():
-            cleanup_termux_install()
+            cleanup_termux_integration()
         return
 
     if on_windows():
         typer.echo("AppData will be set up explicity and a content menu item will be added to your Registry.")
-        setup_windows_install()
+        setup_windows_integration()
     elif on_termux():
         typer.echo("Scripts will now be added to the $HOME/.shortcuts/ directory for launching from the Termux Widget.")
-        setup_termux_install(force=upgrade)
+        setup_termux_integration(force=upgrade)
         typer.echo("Update complete.")
         typer.echo(f"\n{get_package_name()} --version")
         typer.secho(f"{get_package_name()} {get_package_version()}", fg=typer.colors.GREEN, bold=True)
         typer.echo("\n")
-        input("Press Enter to exit...") # moved to internal of setup_termux_install()
+        input("Press Enter to exit...") # moved to internal of setup_termux_integration()
 
 
 @app.command()
@@ -433,18 +436,18 @@ def points_export(
     """
     filter_idcs=None # trouble getting multiple points back, suppress for now
     if plant_name is None:
-        plant_name = get_configurable_plant_name()
+        plant_name = get_configurable_default_plant_name()
 
 
     if isinstance(plant_name,str):
-        api_credentials = get_eds_api_credentials(plant_name=plant_name)
+        api_credentials = get_eds_rest_api_credentials(plant_name=plant_name)
 
     if isinstance(plant_name,list):
         typer.echo("")
         typer.echo(f"Multiple plant names provided: {plant_name} ")
         typer.echo("Querying multiple plants at once currently supported.") 
         typer.echo("Defaulting to use the first name.")
-        api_credentials = get_eds_api_credentials(plant_name=plant_name[0])
+        api_credentials = get_eds_rest_api_credentials(plant_name=plant_name[0])
     
     # Use the retrieved credentials to log in to the API, including custom session attributes
     typer.echo("Logging in to session...")
