@@ -7,7 +7,10 @@ import keyring
 from typing import Dict, Set, List
 import typer
 import click.exceptions
+import logging
+import sys
 from pyhabitat import on_termux, on_ish_alpine, interactive_terminal_is_available, tkinter_is_available, web_browser_is_available
+import pyhabitat as ph
 
 # Define a standard configuration path for your package
 CONFIG_PATH = Path.home() / ".pipeline-eds" / "config.json" ## configuration-example
@@ -96,7 +99,7 @@ def _prompt_for_value(prompt_message: str, hide_input: bool) -> str:
             return None
         return value
         
-    elif tkinter_is_available():
+    elif tkinter_is_available() and False:
         # 2. GUI Mode (Non-interactive fallback)
         from pipeline.guiconfig import gui_get_input
         typer.echo(f"\n --- Non-interactive process detected. Opening GUI prompt. --- ")
@@ -123,12 +126,46 @@ def _prompt_for_value(prompt_message: str, hide_input: bool) -> str:
 
     # If all other options fail
     raise CredentialsNotFoundError(
-        f"Configuration for '{prompt_message}' is missing and "
-        f"neither an interactive terminal, GUI display, nor a web browser is available. "
+        f"Configuration for '{prompt_message}' is missing or cancelled. "
+        f"The program would cancel itself if neither an interactive terminal, nor a GUI display, nor a web browser is available. "
         f"Please use a configuration utility or provide the value programmatically."
     )
 
-def _get_config_with_prompt(config_key: str, prompt_message: str, overwrite: bool = False) -> str:
+def get_temporary_input(prompt_message: str | None) -> str|None :
+    """
+    Seek a temporary user input value, not to be stored.
+    Input options include console, pop-up window, or web browser input field, depending on what is available.
+    """ 
+    typer.echo(f"\n --- One-time temporary value required --- ")
+    typer.echo("You may cancel the input to avoid entering a value.")
+
+    try:
+        new_value = _prompt_for_value(
+            prompt_message=prompt_message,
+            hide_input=False
+        )
+    except (KeyboardInterrupt, EOFError):
+        typer.echo("\n⚠️  Configuration prompt cancelled.")
+        return None
+    except Exception as e:
+        typer.echo(f"\n⚠️  Unexpected error during prompt: {e}")
+        return None
+    if new_value is None:
+        typer.echo("⚠️  No input provided. Returning None.")
+        return None
+    elif new_value == "":
+        if ph.interactive_terminal_is_available() and not typer.confirm("⚠️  An empty string was provided. Are you sure about this?", default=True):
+            return None
+    # Normalize user input for empty-string passwords
+    elif new_value in ("''", '""'):
+        new_value = ""
+    return new_value
+
+def _get_config_with_prompt(config_key: str, 
+                            prompt_message: str, 
+                            overwrite: bool = False, 
+                            forget: bool = False,
+                            ) -> str | None:
     """
     Retrieves a config value from a local file, prompting the user and saving it if missing.
     
@@ -137,15 +174,37 @@ def _get_config_with_prompt(config_key: str, prompt_message: str, overwrite: boo
         prompt_message: The message to display if prompting is needed.
         overwrite: If True, the function will always prompt for a new value,
                    even if one already exists.
-    """
-    config = {}
-    if CONFIG_PATH.exists():
-        with open(CONFIG_PATH, "r") as f:
-            config = json.load(f)
+        forget: If True, credentials should not be stored to system keyring, 
+                and configs should not be stored to program-wide plaintext stored in AppData. 
 
+    
+    Returns:
+        The configuration value (str) if successful, or None if the user cancels.
+    
+    """
+
+    config = {}
+
+     # --- Load existing config file safely ---
+    if CONFIG_PATH.exists():
+        try:
+            with open(CONFIG_PATH, "r") as f:
+                config = json.load(f)
+        except json.JSONDecodeError:
+            typer.echo(f"⚠️  Warning: Existing config file '{CONFIG_PATH.name}' is corrupted.")
+            if not typer.confirm("Do you want to reset the corrupted file to an empty file?", default=False):
+                typer.echo("-> Keeping existing corrupted file. Returning None for the requested value.")
+                return None
+            typer.echo("-> Resetting config file...")
+            config = {}
+        except Exception as e:
+            typer.echo(f"⚠️  Failed to read config file: {e}")
+            return None
+        
     # Get the value from the config file, which will be None if not found
     value = config.get(config_key)
     
+    # --- Handle existing value and overwrite logic ---
     # Check if a value exists and if the user wants to be sure about overwriting
     if value is not None and overwrite:
         typer.echo(f"\nValue for '{prompt_message}' is already set:")
@@ -154,29 +213,59 @@ def _get_config_with_prompt(config_key: str, prompt_message: str, overwrite: boo
             typer.echo("-> Keeping existing value.")
             return value
 
+    # --- Prompt for new value if needed ---
     # If the value is None (not found), or if a confirmation to overwrite was given,
-    # prompt for a new value.
-    
+    # prompt for a new value
     if value is None or overwrite:
         typer.echo(f"\n --- One-time configuration required --- ")
+        typer.echo("You may cancel the input to avoid entering a value.")
 
-        new_value = _prompt_for_value(
-            prompt_message=prompt_message, 
-            hide_input=False
-        )
+        try:
+            new_value = _prompt_for_value(
+                prompt_message=prompt_message,
+                hide_input=False
+            )
+        except (KeyboardInterrupt, EOFError):
+            typer.echo("\n⚠️  Configuration prompt cancelled.")
+            return None
+        except Exception as e:
+            typer.echo(f"\n⚠️  Unexpected error during prompt: {e}")
+            return None
+        if new_value is None:
+            typer.echo("⚠️  No input provided. Configuration not saved.")
+            return None
+        # Normalize user input for empty-string passwords
+        elif new_value in ("''", '""'):
+            new_value = ""
+        elif new_value == "":
+            if ph.interactive_terminal_is_available() and not typer.confirm("⚠️  An empty string was provided. Are you sure about this?", default=True):
+                return None
         
+        # --- Save updated configuration safely ---
         # Save the new value back to the file
-        CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
-        config[config_key] = new_value
-        with open(CONFIG_PATH, "w") as f:
-            json.dump(config, f, indent=4)
-        typer.echo("Configuration stored.")
+        if forget:
+            return new_value
+        try:
+            CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
+            config[config_key] = new_value
+            with open(CONFIG_PATH, "w") as f:
+                json.dump(config, f, indent=4)
+            typer.echo("Configuration stored.")
+        except Exception as e:
+            typer.echo(f"⚠️  Failed to save configuration: {e}")
+            return None
         return new_value
     
-    # If a value existed and overwrite was False, simply return the existing value.
+    # --- Return existing value if no overwrite requested ---
     return value
 
-def _get_credential_with_prompt(service_name: str, item_name: str, prompt_message: str, hide_password: bool = True, overwrite: bool = False) -> str:
+def _get_credential_with_prompt(service_name: str, 
+                                item_name: str, 
+                                prompt_message: str, 
+                                hide: bool = False, 
+                                overwrite: bool = False,
+                                forget: bool = False,
+                                ) -> str | None:
     """
     Retrieves a secret from the keyring, prompting the user and saving it if missing.
     
@@ -184,9 +273,25 @@ def _get_credential_with_prompt(service_name: str, item_name: str, prompt_messag
         service_name: The keyring service name.
         item_name: The credential key.
         prompt_message: The message to display if prompting is needed.
-        hide_password: True if the input should be hidden (getpass), False otherwise (input).
+        hide: True if the input should be hidden (getpass), False otherwise (input).
         overwrite: If True, the function will always prompt for a new credential,
                    even if one already exists.
+        forget: If True, credentials should not be sotred to system keyring, 
+                and configs should not be stored to program-wide plaintext stored in AppData. 
+
+    Examples:
+        OVERWRITE = False
+        HIDEPASSWORD = False 
+
+        service_name: service_name = f"pipeline-eds-db-{plant_name}"
+        item_name: item_name = "username"
+        prompt_message: prompt_message = f"Enter the IP address for your SOAP API (e.g., XXX.XX.X.XXX)" # no colon necessary, managed internally.
+        hide_input: hide_input = HIDEPASSWORD 
+        overwrite: overwrite = OVERWRITE
+
+    Returns:
+        The configuration value (str) if successful, or None if the user cancels.
+    
     """
 
     credential = keyring.get_password(service_name, item_name)
@@ -194,7 +299,7 @@ def _get_credential_with_prompt(service_name: str, item_name: str, prompt_messag
     # Check if a credential exists and if the user wants to be sure about overwriting
     if credential is not None and overwrite:
         typer.echo(f"\nCredential for '{prompt_message}' already exists:")
-        if hide_password:
+        if hide:
             typer.echo(f"  '***'")
         else:
             typer.echo(f"  '{credential}'")
@@ -205,21 +310,49 @@ def _get_credential_with_prompt(service_name: str, item_name: str, prompt_messag
 
     # If the credential is None (not found), or if a confirmation to overwrite was given,
     # prompt for a new value.
-    if credential is None or overwrite:
+    if credential is None or overwrite: 
+        # -- Assign value of new_credential --
+        try:
+            print("Trying to prompt for credential ...")
+            new_credential = _prompt_for_value(
+                prompt_message=prompt_message, 
+                hide_input=hide
+            )
+            print("Success: prompted for credential.")
+            print(f"new_credential = {new_credential}")
+        except (KeyboardInterrupt, EOFError):
+            typer.echo("\n⚠️  Credential prompt cancelled.")
+            return None
+        except Exception as e:
+            typer.echo(f"\n⚠️  Unexpected error during credential prompt: {e}")
+            return None
 
-        new_credential = _prompt_for_value(
-            prompt_message=prompt_message, 
-            hide_input=hide_password
-        )
+        if new_credential is None:
+            typer.echo("⚠️  No input provided. Credential not saved.")
+            return None
+        elif new_credential == "":
+            if ph.interactive_terminal_is_available() and not typer.confirm("⚠️  An empty string was provided. Are you sure about this?", default=True):
+                return None
+        # Normalize user input for empty-string passwords
+        if new_credential in ("''", '""'):
+            new_credential = ""
+        credential = new_credential
+        
+
+        if forget:
+            return credential
             
-        # Store the new credential
-        if new_credential == "''" or new_credential == '""':
-            new_credential = str("") # ensure empty string if user types '' or "" 
-        keyring.set_password(service_name, item_name, new_credential) ## configuration-example
-        typer.echo("Credential stored securely.")
-        return new_credential
-    
-    # If a credential existed and overwrite was False, simply return the existing value.
+        if not forget:
+            # Store the new credential to keyring
+            try:
+                keyring.set_password(service_name, item_name, new_credential)
+            except Exception as e:
+                typer.echo(f"⚠️  Failed to store credential: {e}")
+                return None
+
+        typer.echo("✅  Credential stored securely.")
+
+    # Return existing credential if no overwrite
     return credential
     
 def get_configurable_idcs_list(plant_name: str, overwrite: bool = False) -> List[str]:
@@ -236,7 +369,7 @@ def get_configurable_idcs_list(plant_name: str, overwrite: bool = False) -> List
         f"(e.g., M100FI FI8001 M310LI)"
     )
     
-    idcs_value = _get_config_with_prompt(service_name, prompt_message, overwrite=overwrite)
+    idcs_value = _get_config_with_prompt(config_key = service_name, prompt_message = prompt_message, overwrite=overwrite)
     
     if not idcs_value:
         return []
@@ -255,18 +388,18 @@ def get_configurable_idcs_list(plant_name: str, overwrite: bool = False) -> List
     
     return idcs_list
     
-def get_eds_db_credentials(plant_name: str, overwrite: bool = False) -> Dict[str, str]: # generalized for stiles and maxson
+def get_eds_local_db_credentials(plant_name: str, overwrite: bool = False) -> Dict[str, str]: # generalized for stiles and maxson
     """Retrieves all credentials and config for Stiles EDS Fallback DB, prompting if necessary."""
     service_name = f"pipeline-eds-db-{plant_name}"
 
     # 1. Get non-secret configuration from the local file
-    port = _get_config_with_prompt("eds_db_port", "Enter EDS DB Port (e.g., 3306)")
-    storage_path = _get_config_with_prompt("eds_db_storage_path", "Enter EDS database SQL storage path on your system (e.g., 'E:/SQLData/stiles')")
-    database = _get_config_with_prompt("eds_db_database", "Enter EDS database name on your system (e.g., stiles)")
+    port = _get_config_with_prompt(config_key = "eds_db_port", prompt_message = "Enter EDS DB Port (e.g., 3306)")
+    storage_path = _get_config_with_prompt(config_key = "eds_db_storage_path", prompt_message = "Enter EDS database SQL storage path on your system (e.g., 'E:/SQLData/stiles')")
+    database = _get_config_with_prompt(config_key = "eds_db_database", prompt_message = "Enter EDS database name on your system (e.g., stiles)")
 
     # 2. Get secrets from the keyring
-    username = _get_credential_with_prompt(service_name, "username", "Enter your EDS system username (e.g. root)", hide_password=False, overwrite=overwrite)
-    password = _get_credential_with_prompt(service_name, "password", "Enter your EDS system password (e.g. Ovation1)", hide_password=True, overwrite=overwrite)
+    username = _get_credential_with_prompt(service_name = service_name, item_name = "username", prompt_message = "Enter your EDS system username (e.g. root)", hide=False, overwrite=overwrite)
+    password = _get_credential_with_prompt(service_name = service_name, item_name = "password", prompt_message = "Enter your EDS system password (e.g. Ovation1)", hide=True, overwrite=overwrite)
 
     return {
         'username': username,
@@ -279,31 +412,65 @@ def get_eds_db_credentials(plant_name: str, overwrite: bool = False) -> Dict[str
     }
 
 
-def get_configurable_plant_name(overwrite=False) -> str:
+def get_configurable_default_plant_name(overwrite=False) -> str :
     '''Comma separated list of plant names to be used as the default if none is provided in other commands.'''
-    plant_name = _get_config_with_prompt(f"configurable_plantname_eds_api", f"Enter plant name(s) to be used as the default", overwrite=overwrite)
+    plant_name = _get_config_with_prompt(config_key = f"configurable_plantname_eds_api", prompt_message = f"Enter plant name(s) to be used as the default", overwrite=overwrite)
     if ',' in plant_name:
         plant_names = plant_name.split(',')
         return plant_names
     else:
         return plant_name
 
-def get_eds_api_credentials(plant_name: str, overwrite: bool = False) -> Dict[str, str]:
+def get_eds_rest_api_credentials(plant_name: str, overwrite: bool = False, forget: bool = False) -> Dict[str, str]:
     """Retrieves API credentials for a given plant, prompting if necessary."""
+
+    from pipeline.api.eds import EdsClient
+
     service_name = f"pipeline-eds-api-{plant_name}"
-    
-    #url = _get_config_with_prompt(f"{plant_name}_eds_api_url", f"Enter {plant_name} API URL (e.g., http://000.00.0.000:43084/api/v1)", overwrite=overwrite)
-    url = _get_eds_url_config_with_prompt(f"{plant_name}_eds_api_url", f"Enter {plant_name} API URL (e.g., http://000.00.0.000:43084/api/v1, or just 000.00.0.000)", overwrite=overwrite)
-    username = _get_credential_with_prompt(service_name, "username", f"Enter your API username for {plant_name} (e.g. admin)", hide_password=False, overwrite=overwrite)
-    password = _get_credential_with_prompt(service_name, "password", f"Enter your API password for {plant_name} (e.g. '')", overwrite=overwrite)
-    idcs_to_iess_suffix = _get_config_with_prompt(f"{plant_name}_eds_api_iess_suffix", f"Enter iess suffix for {plant_name} (e.g., .UNIT0@NET0)", overwrite=overwrite)
-    zd = _get_config_with_prompt(f"{plant_name}_eds_api_zd", f"Enter {plant_name} ZD (e.g., 'Maxson' or 'WWTF')", overwrite=overwrite)
+    overwrite = False
+    #url = _get_config_with_prompt(config_key =f"{plant_name}_eds_api_url", prompt_message = f"Enter {plant_name} API URL (e.g., http://000.00.0.000:43084/api/v1)", overwrite=overwrite)
+    #url = _get_eds_url_config_with_prompt(config_key = f"{plant_name}_eds_api_url", prompt_message = f"Enter {plant_name} EDS API URL (e.g., http://000.00.0.000:43084/api/v1, or just 000.00.0.000)", overwrite=overwrite)
+    eds_base_url = _get_credential_with_prompt(service_name = service_name, item_name = f"{plant_name}_eds_base_url", prompt_message =  f"Enter {plant_name} EDS base url (e.g., http://000.00.0.000, or just 000.00.0.000)", overwrite=overwrite)
+    eds_base_url = _get_base_url_config_with_prompt(service_name = f"{plant_name}_eds_base_url", prompt_message = f"Enter {plant_name} EDS base url (e.g., http://000.00.0.000, or just 000.00.0.000)")
+    eds_rest_api_port = _get_config_with_prompt(config_key = f"{plant_name}_eds_rest_api_port", prompt_message = f"Enter {plant_name} EDS REST API port (e.g., 43084)", overwrite=overwrite)
+    eds_rest_api_sub_path = _get_config_with_prompt(config_key = f"{plant_name}_eds_rest_api_sub_path", prompt_message = f"Enter {plant_name} EDS REST API sub path (e.g., 'api/v1')", overwrite=overwrite)
+    eds_soap_api_port = _get_config_with_prompt(config_key = f"{plant_name}_eds_soap_api_port", prompt_message = f"Enter {plant_name} EDS SOAP API port (e.g., 43080)", overwrite=overwrite)
+    eds_soap_api_sub_path = _get_config_with_prompt(config_key = f"{plant_name}_eds_soap_api_sub_path", prompt_message = f"Enter {plant_name} EDS SOAP API WSDL path (e.g., 'eds.wsdl')", overwrite=overwrite)
+    username = _get_credential_with_prompt(service_name = service_name, item_name = "username", prompt_message = f"Enter your EDS API username for {plant_name} (e.g. admin)", hide=False, overwrite=overwrite)
+    password = _get_credential_with_prompt(service_name = service_name, item_name = "password", prompt_message = f"Enter your EDS API password for {plant_name} (e.g. '')", overwrite=overwrite)
+    idcs_to_iess_suffix = _get_config_with_prompt(config_key = f"{plant_name}_eds_api_iess_suffix", prompt_message = f"Enter iess suffix for {plant_name} (e.g., .UNIT0@NET0)", overwrite=overwrite)
+    zd = _get_config_with_prompt(config_key = f"{plant_name}_eds_api_zd", prompt_message = f"Enter {plant_name} ZD (e.g., 'Maxson' or 'WWTF')", overwrite=overwrite)
     
     #if not all([username, password]):
     #    raise CredentialsNotFoundError(f"API credentials for '{plant_name}' not found. Please run the setup utility.")
-        
+    eds_rest_api_sub_path = str(eds_rest_api_sub_path).rstrip("/").lstrip("/").replace(r"\\","/").lower()
+    eds_soap_api_port = int(eds_soap_api_port)
+    eds_soap_api_sub_path = eds_soap_api_sub_path
+
+    # EDS REST API Pattern: url = f"http://{url}:43084/api/v1" # assume EDS patterna and port http and append api/v1 if user just puts in an IP
+    
+    # Comparable SOAP API function, for documentation:
+    eds_soap_api_url = EdsClient.get_soap_api_url(base_url = eds_base_url,
+                                                    eds_soap_api_port = str(eds_soap_api_port),
+                                                    eds_soap_api_sub_path = eds_soap_api_sub_path)
+    if eds_soap_api_url is None:
+        logging.info("Not enough information provided to build: eds_soap_api_url.")
+        logging.info("Please rerun your last command or try something else.")
+        sys.exit()
+    
+    
+    eds_rest_api_url = EdsClient.get_rest_api_url(eds_base_url, 
+                                        str(eds_rest_api_port),
+                                        eds_rest_api_sub_path
+                                        ) 
+    
+    if eds_rest_api_url is None:
+        logging.info("Not enough information provided to build: eds_rest_api_url.")
+        logging.info("Please rerun your last command or try something else.")
+        sys.exit()
+
     return {
-        'url': url,
+        'url': eds_rest_api_url,
         'username': username,
         'password': password,
         'zd': zd,
@@ -320,10 +487,10 @@ def get_external_api_credentials(party_name: str, overwrite: bool = False) -> Di
     The RJN API client was the first external API client, and it uses the term 'client_id' in place of the term 'username'."""
     service_name = f"pipeline-external-api-{party_name}"
     
-    url = _get_config_with_prompt(service_name, f"Enter {party_name} API URL (e.g., http://api.example.com)", overwrite=overwrite)
-    username = _get_credential_with_prompt(service_name, "username", f"Enter the username AKA client_id for the {party_name} API",hide_password=False, overwrite=overwrite)
-    #client_id = _get_credential_with_prompt(service_name, "client_id", f"Enter the client_id for the {party_name} API",hide_password=False, overwrite=overwrite)
-    password = _get_credential_with_prompt(service_name, "password", f"Enter the password for the {party_name} API", overwrite=overwrite)
+    url = _get_config_with_prompt(config_key = service_name, prompt_message = f"Enter {party_name} API URL (e.g., http://api.example.com)", overwrite=overwrite)
+    username = _get_credential_with_prompt( service_name = service_name, item_name = "username", prompt_message = f"Enter the username AKA client_id for the {party_name} API",hide=False, overwrite=overwrite)
+    #client_id = _get_credential_with_prompt(service_name = service_name, item_name = "client_id", prompt_message = f"Enter the client_id for the {party_name} API",hide=False, overwrite=overwrite)
+    password = _get_credential_with_prompt(service_name = service_name, item_name = "password", prompt_message = f"Enter the password for the {party_name} API", overwrite=overwrite)
 
     client_id = username
     
@@ -368,12 +535,18 @@ def _is_likely_ip(url: str) -> bool:
     for part in parts:
         if not part.isdigit() or not (0 <= int(part) <= 255):
             return False
-    return True
+    return True    
 
-def _get_eds_url_config_with_prompt(config_key: str, prompt_message: str, overwrite: bool = False) -> str:
-    url = _get_config_with_prompt(config_key, prompt_message, overwrite=overwrite)
+def _get_base_url_config_with_prompt(service_name: str, 
+                                         prompt_message: str, 
+                                         overwrite: bool = False
+                                         ) -> str:
+    #url = _get_config_with_prompt(config_key = service_name, prompt_message = prompt_message, overwrite=overwrite)
+    url = _get_credential_with_prompt(service_name=service_name, item_name="base_url",prompt_message=prompt_message, overwrite=overwrite)
+    if url is None:
+        return None
     if _is_likely_ip(url):
-        url = f"http://{url}:43084/api/v1" # assume EDS patterna and port http and append api/v1 if user just put in an IP
+        url = f"http://{url}" # assume EDS patterns and port http if user just puts in an IP
     return url
 
 class CredentialsNotFoundError(Exception):
@@ -381,7 +554,7 @@ class CredentialsNotFoundError(Exception):
     pass
 
 # Example usage in your main pipeline
-def frontload_build_all_credentials():
+def frontload_build_all_credentials(forget : bool = False):
     """
     Sets up all possible API and database credentials for the pipeline.
     
@@ -394,12 +567,21 @@ def frontload_build_all_credentials():
     
     Note: This will prompt for credentials for all supported plants and external
     APIs in sequence.
+
+    'forget' should be an option that can be toggled with env var PIPELINE_FORGET, which has a default of False when set up.
+    Functions that require forget:
+        - frontload_build_all_credentials()
+        - _get_config_with_prompt() # stored as plaintext
+        - _get_credential_with_prompt() # stored to keyring
+        
+    Functions that store things are antithetical to 'forget':
+        - _get_eds_local_db_credentials() # helper functon
     """
     
     try:
-        maxson_api_creds = get_eds_api_credentials(plant_name = "Maxson")
-        stiles_api_creds = get_eds_api_credentials(plant_name = "Stiles")
-        stiles_db_creds = get_eds_db_credentials(plant_name = "Stiles")
+        maxson_api_creds = get_eds_rest_api_credentials(plant_name = "Maxson")
+        stiles_api_creds = get_eds_rest_api_credentials(plant_name = "Stiles")
+        stiles_db_creds = get_eds_local_db_credentials(plant_name = "Stiles")
         rjn_api_creds = get_external_api_credentials("RJN")
         
         # Now use the credentials normally in your application logic
